@@ -1,6 +1,6 @@
 package TOML::Tiny;
 # ABSTRACT: a minimal, pure perl TOML parser and serializer
-$TOML::Tiny::VERSION = '0.13';
+$TOML::Tiny::VERSION = '0.14';
 use strict;
 use warnings;
 no warnings qw(experimental);
@@ -20,9 +20,18 @@ our @EXPORT = qw(
 # TOML module compatibility
 #-------------------------------------------------------------------------------
 sub from_toml {
-  my $source = shift;
-  my $parser = TOML::Tiny::Parser->new(@_);
-  my $toml = eval{ $parser->parse($source) };
+  my ($source, %param) = @_;
+
+  # strict was previously strict_arrays; accept both for backward
+  # compatibility.
+  if (exists $param{strict_arrays}) {
+    $param{strict} = $param{strict_arrays};
+    delete $param{strict_arrays};
+  }
+
+  my $parser = TOML::Tiny::Parser->new(%param);
+  my $toml   = eval{ $parser->parse($source) };
+
   if (wantarray) {
     return ($toml, $@);
   } else {
@@ -32,7 +41,16 @@ sub from_toml {
 }
 
 sub to_toml {
-  goto \&TOML::Tiny::Writer::to_toml;
+  my ($data, %param) = @_;
+
+  # strict was previously strict_arrays; accept both for backward
+  # compatibility.
+  if (exists $param{strict_arrays}) {
+    $param{strict} = $param{strict_arrays};
+    delete $param{strict_arrays};
+  }
+
+  TOML::Tiny::Writer::to_toml($data, %param);
 }
 
 #-------------------------------------------------------------------------------
@@ -50,9 +68,7 @@ sub decode {
 
 sub encode {
   my ($self, $data) = @_;
-  TOML::Tiny::Writer::to_toml($data,
-    strict_arrays => $self->{strict_arrays},
-  );
+  TOML::Tiny::Writer::to_toml($data, strict => $self->{strict});
 }
 
 #-------------------------------------------------------------------------------
@@ -76,7 +92,7 @@ TOML::Tiny - a minimal, pure perl TOML parser and serializer
 
 =head1 VERSION
 
-version 0.13
+version 0.14
 
 =head1 SYNOPSIS
 
@@ -110,9 +126,8 @@ version 0.13
 </p>
 
 C<TOML::Tiny> implements a pure-perl parser and generator for the
-L<TOML|https://github.com/toml-lang/toml> data format. It conforms to TOML v0.5
-(with a few caveats; see L</strict_arrays>) with support for more recent
-changes in pursuit of v1.0.
+L<TOML|https://github.com/toml-lang/toml> data format. It conforms to TOML v1.0
+(with a few caveats; see L</strict>).
 
 C<TOML::Tiny> strives to maintain an interface compatible to the L<TOML> and
 L<TOML::Parser> modules, and could even be used to override C<$TOML::Parser>:
@@ -144,11 +159,6 @@ errors will result in returning two values, C<undef> and an error message.
 
   my ($result, $error) = from_toml($toml_string);
 
-Homogenous array strictures are enabled by passing C<strict_arrays>:
-
-  # Croaks
-  my $result = from_toml(q{mixed=[1, 2, "three"]})
-
 Additional arguments may be passed after the toml source string; see L</new>.
 
 =head3 GOTCHAS
@@ -157,7 +167,7 @@ Additional arguments may be passed after the toml source string; see L</new>.
 
 =item Big integers and floats
 
-C<TOML> supports integers and floats larger than what many perls support.  when
+C<TOML> supports integers and floats larger than what many perls support. When
 C<TOML::Tiny> encounters a value it may not be able to represent as a number,
 it will instead return a L<Math::BigInt> or L<Math::BigFloat>. This behavior
 can be overridden by providing inflation routines:
@@ -179,11 +189,6 @@ Encodes a hash ref as a C<TOML>-formatted string.
   # [foo]
   # bar="bat"
 
-Homogenous array strictures are enabled by passing C<strict_arrays>:
-
-  # Croaks
-  my $toml = to_toml({mixed => [1, 2, "three"]}, strict_arrays => 1);
-
 =head3 mapping perl to TOML types
 
 =head4 table
@@ -330,16 +335,14 @@ routine.
     };
   );
 
-=item strict_arrays
+=item strict
 
-C<TOML v0.5> specified homogenous arrays. This has since been removed and will no
-longer be part of the standard as of C<v1.0> (as of the time of writing; the
-author of C<TOML> has gone back and forth on the issue, so no guarantees).
+C<strict> imposes some miscellaneous strictures on C<TOML> input, such as
+disallowing trailing commas in inline tables and failing on invalid UTF8 input.
 
-By default, C<TOML::Tiny> is flexible and supports heterogenous arrays. If you
-wish to require strictly typed arrays (for C<TOML>'s definition of "type",
-anyway), C<strict_arrays> will produce an error when encountering arrays with
-heterogenous types.
+B<Note:> C<strict> was previously called C<strict_arrays>. Both are accepted
+for backward compatibility, although enforcement of homogenous arrays is no
+longer supported as it has been dropped from the spec.
 
 =back
 
@@ -349,8 +352,7 @@ Decodes C<TOML> and returns a hash ref. Dies on parse error.
 
 =head2 encode
 
-Encodes a perl hash ref as a C<TOML>-formatted string. Dies when encountering
-an array of mixed types if C<strict_arrays> was set.
+Encodes a perl hash ref as a C<TOML>-formatted string.
 
 =head2 parse
 
@@ -363,13 +365,11 @@ C<TOML::Tiny> differs in a few significant ways from the L<TOML> module,
 particularly in adding support for newer C<TOML> features and strictness.
 
 L<TOML> defaults to lax parsing and provides C<strict_mode> to (slightly)
-tighten things up. C<TOML::Tiny> defaults to (somehwat) stricter parsing, with
-the exception of permitting heterogenous arrays (illegal in v4 and v0.5, but
-permissible in the upcoming v1.0); optional enforcement of homogenous arrays is
-supported with C<strict_arrays>.
+tighten things up. C<TOML::Tiny> defaults to (somehwat) stricter parsing,
+enabling some extra strictures with L</strict>.
 
 C<TOML::Tiny> supports a number of options which do not exist in L<TOML>:
-L</inflate_integer>, L</inflate_float>, and L</strict_arrays>.
+L</inflate_integer>, L</inflate_float>, and L</strict>.
 
 C<TOML::Tiny> ignores invalid surrogate pairs within basic and multiline
 strings (L<TOML> may attempt to decode an invalid pair). Additionally, only
